#!/bin/sh
##############################################################
# This script helps you to setup some virtual xen-mashines.
# Until now it isn't very robust so handle whith care
# This Version is for the stock-kernels of XEN 3.0.0
#
# 19.Jan.2006 - mail@matthias-qua.de
##############################################################

failed=0
check="CHECK-INSTALL"

echo "Xen-Install-Script $(check) dependencycheck "
for f in check_* ; do
    case $f in
        *~)
            continue
            ;;
        *)
            ;;
    esac
    if ! [ -x $f ] ; then
        continue
    fi
    if ! grep -q ${check} $f ; then
        continue
    fi
    echo -n "Checking $f: "
    if ./$f 2>&1 ; then
        echo OK
    else
	echo FAILED
	echo "Please install all dependencies"
	exit 1
    fi
done


if [ "$#" -lt 3 ]; then
    echo "Usage: make_xen [vm_count] [vm_memsize] [vm_hddsize] <vm_swapsize>"
    echo "All sizes are interpreted as megabytes"
	echo ""
    echo "Memory should not be less than 32MB and HDD not less than 250MB"
    exit
fi

apt-get update
apt-get install iproute python python-twisted bridge-utils curl debootstrap ssh libcurl3-dev

XEN_INSTALL_STATUS=0
if [ -e tmp/xen_inst_status ]; then
    XEN_INSTALL_STATUS=`cat /tmp/xen_inst_status`
fi

XEN_IMAGE_DIR=/var/xen_images
XEN_NUMBER=$1
XEN_VM_MEM=$2
XEN_VM_HDDSIZE=$3
if [ "$#" != 4 ]; then
    XEN_VM_SWAPSIZE=0
else
    XEN_VM_SWAPSIZE=$4
fi
XEN_U_KERNEL=vmlinuz-2.6-xenU

if [ ! -e "$XEN_IMAGE_DIR" ]; then
    mkdir -p $XEN_IMAGE_DIR
fi

if [ "$XEN_INSTALL_STATUS" -lt 1 ]; then

    echo "================================================================================"
    echo "Building diskimages, this may take some time."
    echo "================================================================================"
    declare -i i=1
    while [ $i -le "$XEN_NUMBER" ]; do
	echo "================================================================================"
	echo "Building disk no. ${i}"
	echo "================================================================================"
	dd if=/dev/zero of=$XEN_IMAGE_DIR/xen_vm${i}_disk bs=1024k count=$XEN_VM_HDDSIZE
	mkfs.ext3 -F $XEN_IMAGE_DIR/xen_vm${i}_disk
	if [ "$XEN_VM_SWAPSIZE" -gt 0 ]; then
	    dd if=/dev/zero of=$XEN_IMAGE_DIR/xen_vm${i}_swap bs=1024k count=$XEN_VM_SWAPSIZE
	    mkswap -F $XEN_IMAGE_DIR/xen_vm${i}_swap
	fi
	
	if [ ! -e /mnt/xen_vm${i}_disk ]; then
	    mkdir /mnt/xen_vm${i}_disk
	fi

	echo "#This startupscript was automaticaly buildet by make_xen.sh" >/etc/xen/xen_vm${i}
	echo "#If you want to have some extra options see /etc/xen/xmexample{1|2}"  >>/etc/xen/xen_vm${i}
	echo ""                                       >>/etc/xen/xen_vm${i}
	echo "kernel = \"/boot/$XEN_U_KERNEL\""       >>/etc/xen/xen_vm${i}
	echo "memory = $XEN_VM_MEM"                   >>/etc/xen/xen_vm${i}
	echo "name = \"xen_vm${i}\""                  >>/etc/xen/xen_vm${i}
	if [ "$XEN_VM_SWAPSIZE" -gt 0 ]; then
	    echo "disk = [ 'file:$XEN_IMAGE_DIR/xen_vm${i}_disk,sda1,w', 'file:$XEN_IMAGE_DIR/xen_vm${i}_swap,sda2,w'  ]" >>/etc/xen/xen_vm${i}
	else	
	    echo "disk = [ 'file:$XEN_IMAGE_DIR/xen_vm${i}_disk,sda1,w' ]" >>/etc/xen/xen_vm${i}
	fi
	echo "root = \"/dev/sda1 ro\""               >>/etc/xen/xen_vm${i}
	echo "nics = 1"                              >>/etc/xen/xen_vm${i}
	echo "dhcp = \"off\""                        >>/etc/xen/xen_vm${i}
	echo "ip = \"192.168.2.5${i}\""              >>/etc/xen/xen_vm${i}
	echo "netmask = \"255.255.255.0\""           >>/etc/xen/xen_vm${i}
	echo "gateway = \"192.168.2.1\""             >>/etc/xen/xen_vm${i}
	echo "hostname = \"xen_vm_1\""               >>/etc/xen/xen_vm${i}

	mount -o loop $XEN_IMAGE_DIR/xen_vm${i}_disk /mnt/xen_vm${i}_disk
	i=$i+1
    done
    echo "1" > /tmp/xen_install_status
else
    echo "Images have already been crated. Skipping"

fi

if [ "$XEN_INSTALL_STATUS" -lt 2 ]; then

    echo "================================================================================"
    echo "All diskimages are created."
    echo "I will now download Debian (Sarge) to the first disk image (appr. 40 MB)"
    echo "Dont't panic... The output isn't very verbose. Mostly it finshes cleanly :-P"
    echo "================================================================================"
    debootstrap --verbose --arch i386 sarge /mnt/xen_vm1_disk http://user.informatik.uni-goettingen.de/mirror/debian/
    echo "================================================================================"
    echo "Download and installation is done. Now I will configure your first VM"
    echo "================================================================================"

    echo "2" > /tmp/xen_install_status
else
    echo "Bootstrap image already installed. Skipping"
fi

if [ "$XEN_INSTALL_STATUS" -lt 3 ]; then
    echo "Configuring the network..."
    echo "auto lo"                       >/mnt/xen_vm1_disk/etc/network/interfaces
    echo "iface lo inet loopback"        >>/mnt/xen_vm1_disk/etc/network/interfaces
    echo ""                              >>/mnt/xen_vm1_disk/etc/network/interfaces
    echo "auto eth0"                     >>/mnt/xen_vm1_disk/etc/network/interfaces
    echo "iface eth0 inet static"        >>/mnt/xen_vm1_disk/etc/network/interfaces
    echo "   address 192.168.2.51"       >>/mnt/xen_vm1_disk/etc/network/interfaces
    echo "   netmask 255.255.255.0"      >>/mnt/xen_vm1_disk/etc/network/interfaces
    echo "   network 192.168.2.0"        >>/mnt/xen_vm1_disk/etc/network/interfaces
    echo "   broadcast 192.168.2.255"    >>/mnt/xen_vm1_disk/etc/network/interfaces
    echo "   gateway 192.168.2.1"        >>/mnt/xen_vm1_disk/etc/network/interfaces

    echo "Hostname ist set to xen_vm1"
    echo "xen_vm1"                       >/mnt/xen_vm1_disk/etc/hostname

    echo "Apt-Sources have been set to uni-goettingen.de"
    echo "deb http://user.informatik.uni-goettingen.de/mirror/debian/ stable main non-free contrib" >/mnt/xen_vm1_disk/etc/apt/sources.list
    echo "deb-src http://user.informatik.uni-goettingen.de/mirror/debian/ stable main non-free contrib" >>/mnt/xen_vm1_disk/etc/apt/sources.list 
    echo "deb http://security.debian.org/ stable/updates main" >>/mnt/xen_vm1_disk/etc/apt/sources.list

# Filesystem
    echo "/dev/sda1       /       ext3    errors=remount-ro       0       1" >/mnt/xen_vm1_disk/etc/fstab
    if [ "$XEN_VM_SWAPSIZE" -gt 0 ]; then
	echo "/dev/sda2       none    swap    sw                      0       0" >>/mnt/xen_vm1_disk/etc/fstab
    fi
    echo "proc            /proc   proc    defaults                0       0" >>/mnt/xen_vm1_disk/etc/fstab
    echo "sys             /sys    sys     defaults                0       0" >>/mnt/xen_vm1_disk/etc/fstab

# Deactivate tls
    mv /mnt/xen_vm1_disk/lib/tls /mnt/xen_vm1_disk/lib/tls.disabled

# Personalshit
    cp -f /root/.bashrc /root/.profile /mnt/xen_vm1_disk/root

    echo "3" > /tmp/xen_install_status
else
    echo "Xen has already been configured. Skipping"
fi

if [ "$XEN_INSTALL_STATUS" -lt 4 ]; then
    
    echo "================================================================================"
    echo "I will now chroot into your new system and run the standart Debian scripts"
    echo "================================================================================"
    chroot /mnt/xen_vm1_disk apt-get update 
    chroot /mnt/xen_vm1_disk apt-get install localeconf mc fonty ssh lynx screen bzip2
    chroot /mnt/xen_vm1_disk base-config
fi

umount /mnt/xen_vm1_disk
rm -f /tmp/xen_install_status

echo "================================================================================"
echo "Now it's time to run and configure your VM"
echo "So start up you XEN-Instance by doing a
echo "# xm create - /etc/xen/xen_vm1
echo "Good luck && crossing fingers :-P
echo "================================================================================"
